package cn.com.qmth.stmms.ms.log;

import cn.com.qmth.stmms.ms.core.domain.MarkLog;
import cn.com.qmth.stmms.ms.core.domain.user.MarkUser;
import cn.com.qmth.stmms.ms.core.repository.MarkLogRepo;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import org.aspectj.lang.JoinPoint;
import org.aspectj.lang.annotation.AfterReturning;
import org.aspectj.lang.annotation.Aspect;
import org.aspectj.lang.annotation.Pointcut;
import org.aspectj.lang.reflect.MethodSignature;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpSession;
import java.util.Arrays;
import java.util.Objects;

/**
 * @Description: 纪检审查aop
 * @Param:
 * @return:
 * @Author: wangliang
 * @Date: 2019/10/19
 */
@Aspect
@Component
public class MarkLogAop {

    private static Logger LOGGER = LoggerFactory.getLogger(MarkLogAop.class);

    @Autowired
    private MarkLogRepo markLogRepo;

//    @Autowired
//    private MarkUserRepo markUserRepo;
//
//    @Autowired
//    private PaperRepo paperRepo;
//
//    @Autowired
//    private LoginConfig loginConfig;

    /**
     * 分档切入点
     */
    @Pointcut("execution(* cn.com.qmth.stmms.ms.marking.api.MarkTaskApi.marking(..))")
    public void getMarkAspect() {
    }

    /**
     * 评卷员切入方法
     *
     * @param joinPoint
     */
    @AfterReturning(value = "getMarkAspect()", returning = "result")
    public void afterReturnMarkPoint(JoinPoint joinPoint, Object result) throws Exception {
        String className = joinPoint.getSignature().getDeclaringTypeName();
        String methodName = joinPoint.getSignature().getName();
        LOGGER.info("mark aspect afterReturnMarkPoint path:{}", className + "." + methodName);
        ServletRequestAttributes attributes = (ServletRequestAttributes) RequestContextHolder.getRequestAttributes();
        HttpServletRequest request = attributes.getRequest();
        HttpSession session = request.getSession();
        String token = (String) session.getAttribute("token");
        if (Objects.isNull(token)) {
            throw new Exception("请先登录");
        }
        MarkUser markUser = (MarkUser) session.getAttribute("markUser");
        LOGGER.info("markUser:{}", JSONObject.toJSONString(markUser));
        Object[] args = joinPoint.getArgs(); // 参数值
        String[] argNames = ((MethodSignature) joinPoint.getSignature()).getParameterNames();//参数名
        JSONArray jsonArgsArray = JSONObject.parseArray(JSONObject.toJSONString(args));
        JSONArray jsonArgNamesArray = JSONObject.parseArray(JSONObject.toJSONString(argNames));
        LOGGER.info("jsonArgsArray:{},jsonArgNamesArray:{}", jsonArgsArray, jsonArgNamesArray);
        MarkLog markLog = new MarkLog();
        markLogRepo.save(markLog);
    }


    /**

     @Pointcut("execution(* cn.com.qmth.stmms.ms.marking.api.PaperApi.marking(..))")
     public void getPaper() {
     }

     @AfterReturning(pointcut = "getPaper()")
     @SuppressWarnings("unchecked") public void afterLevelMarkPaperPoint(JoinPoint joinPoint) {
     Paper paper = (Paper) joinPoint.getArgs()[0];
     HashMap<String,String> body =  (HashMap<String, String>) joinPoint.getArgs()[1];
     String level = body.get("level");
     HttpServletRequest request = ((ServletRequestAttributes) RequestContextHolder
     .getRequestAttributes()).getRequest();
     HttpSession session = request.getSession();
     String token = (String) session.getAttribute("token");
     if (!StringUtils.isEmpty(token)) {
     String loginName = token.substring(14);
     AbstractUser user = null;
     if(loginConfig.getLoginName().equals(loginName)){
     user = new Admin(loginConfig.getLoginName(),loginConfig.getPassword());
     user.setName("系统管理员");
     }
     else{
     user = markUserRepo.findByLoginName(loginName);
     }
     MarkLog markLog = new MarkLog(paper.getWorkId(),
     MarkStage.LEVEL,
     user.getLoginName(),
     user.getName(),
     paper.getExamNumber(),
     paper.getSecretNumber(),
     level);
     markLogRepo.save(markLog);
     }else{
     MarkLog markLog = new MarkLog(paper.getWorkId(),
     MarkStage.LEVEL,
     null,
     "科组长",
     paper.getExamNumber(),
     paper.getSecretNumber(),
     level);
     markLogRepo.save(markLog);
     }
     }
     **/

}
